# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: rtt.rb 115 2008-10-02 13:03:24Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

module Sensors

  # This class implements a round trip time +Sensor+ using fping.
  class RTT < Sensor
    @@nodes = Hash.new()
    @@sensors = Hash.new();

    @cur_rtt = 0
    @cur_lr = 0

    def initialize(node, xmlsensor)
      super(node, xmlsensor, "rtt")
      @@nodes[node.address] = node
      @@sensors[node.address] = self
	  

      RingList.new(node, self, "rtt" , {
		     :lines => 2,
		     :title => 'RTT',
		     :class => @class,
		     :max => [0, 100],
		     :format => ["t", "p"],
		     :type => ["v", "v"],
		     :color => ["Green", "Red"],
		     :legend => ["RTT", "Loss"],
		     :slegend => ["r", "l"],
		   })

      self.add_expr("rtt") { @cur_rtt }
      self.add_expr("lr") { @cur_lr }
    end

    def addProbe(min, avg, max, lr)
      @cur_rtt = avg.to_i
      @cur_lr = lr.to_i

      self.check_mon
      pushProbe([@cur_rtt, (@cur_lr == 0 ? nil : @cur_lr) ])
    end
	
    def self.monitor()
      fping = IO.popen('/usr/bin/fping -Q 5 -Al -p 1200 2>&1', 'w+')
      
      @@nodes.each_key{ |naddr|
	fping.puts(naddr)
      }
      fping.close_write
      
      while line=fping.gets do
	m = line.match(/^([\d.]+)\s*: xmt\/rcv\/%loss = \d+\/\d+\/([\d.]+)%(.*)$/)
	
	next if(m == nil)
	
	host = m[1]
	loss = Integer(m[2])
	
	min = 0
	avg = 0
	max = 0
	
	if rtt=m[3].match(/([\d.]+)\/([\d.]+)\/([\d.]+)/)
	  min = rtt[1]
	  avg = rtt[2]
	  max = rtt[3]
	end
	
	@@sensors[host].addProbe(min, avg, max, loss)
      end
    end
  end
end
