# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: wlc.rb 152 2008-10-23 10:27:12Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

module Sensors
  class WLC < Sensor
    @@sensors = Array.new

    attr_reader :rlname, :lines

    attr_accessor :mon_reachable, :mon_clients

    def initialize(node, xmlsensor)
      @@sensors.push(self)

      @rlname = "wlc"

      @sysLocation_OID = "1.3.6.1.2.1.1.6.0"
      @base_OID = "1.3.6.1.4.1.14179.2.1.1.1"
      @OIDs = Array.new
      @oid2idx = Hash.new

      @mon_clients = Array.new
      @mon_reachable = 1

      super(node, xmlsensor, @rlname)

      begin
	response = node.snmp.get([@sysLocation_OID])
      rescue
	puts "\nERROR: Could not query SNMP: #{$!}"
	exit 1
      end

      location = nil

      response.each_varbind do |vb|
	begin
	    case vb.name.to_s
	    when @sysLocation_OID
		location = vb.value.to_s.strip
	    end
	rescue
	    puts "Error retrieving #{vb.name.to_s}: #{$!}"
	end
      end

      max = Array.new
      format = Array.new
      type = Array.new
      color = Array.new
      legend = Array.new
      slegend = Array.new

      dowhile = true
      @lines = 0
      _colors = ["Blue", "Green", "Orange", "Purple"]
      while dowhile do
	begin
	    response = node.snmp.get(["#{@base_OID}.2.#{@lines+1}"])

	    response.each_varbind do |vb|
		if vb.value == SNMP::NoSuchInstance then
		    dowhile = false
		else
		    ssid = "?"

		    response.each_varbind do |vb|
			ssid = vb.value.to_s
		    end

		    max.push(1)
		    format.push("c")
		    type.push("v")
		    color.push(_colors[@lines % _colors.size])
		    legend.push("#{ssid}")
		    slegend.push(@lines+1)

		    @oid2idx["#{@base_OID}.38.#{@lines+1}"] = @lines
		    @OIDs.push("#{@base_OID}.38.#{@lines+1}")

		    @lines = @lines + 1
		end
	    end
	rescue
	    dowhile = false
	    puts "Error: #{$!}"
	end
      end

      RingList.new(node, self, @rlname , {
		     :lines => @lines,
		     :title => "Clients",
		     :descr => location,
		     :class => @class,
		     :max => max,
		     :format => format,
		     :type => type,
		     :color => color,
		     :legend => legend,
		     :slegend => slegend,
		   })
      self.add_expr("reachable") { @mon_reachable }
      self.add_expr("clients") { @mon_clients }

      @node.addSnmpMon(self, @OIDs)
    end

    def monitor(values)
      if values then
	values.each_pair do |oid, val|
	    if @oid2idx[oid] then
		@mon_clients[@oid2idx[oid]] = val.to_i
	    else
		@mon_clients[@oid2idx[oid]] = nil
	    end
	end

	@mon_reachable = 1
      else
	@mon_reachable = 0
      end

      check_mon

      pushProbe(@mon_clients)

      true
    end
  end
end
