require 'xmlrpc/client'

module Xen

  class Connection

    # Initializes an xml-rpc connection to a Xen host, and logs in with the given credentials.
    # === Example
    # Xen::Connection.new ('xenhost.example.com', 2000, 'login', 'pass')
    def initialize(uri, login = 'user', pass = '')
      @server = XMLRPC::Client.new2(uri)
      @session = @server.call("session.login_with_password", login, pass)
    end

    # Returns the session_id of the connection
    def session_id
      @session['Value']
    end

    def hosts
      h = Array.new
      @server.call("host.get_all", session_id)['Value'].each{|host_uid|
        h.push(Host.new(host_uid, self))
      }
      return h
    end

    # Make a direct call to the Xen API.
    # === Example
    # call('session.get_this_host', session_id) # => {"Status"=>"Success", "Value"=>"3a8f495d-939d-823b-fa08-fc407c73dc94"}
    def call(function_name, *params)
      call_without_session(function_name, session_id, *params)
    end

    def to_s
      session_id.to_s
    end

    private
    def call_without_session(function_name, *params)
      @server.call(function_name, *params)
    end

  end

end
