module Xen

  class Host

    attr_reader :connection

    # Creates a new Host object
    def initialize(uid, connection)
      @uid = uid
      @connection = connection
    end

    # Returns the uid of the host.
    def uid # Lazy Loaded
      @uid ||= get_value("session.get_this_host", @connection.session_id)
    end

    # Gives an array of active VM's on that Host.
    def vms
      get_value("host.get_resident_VMs", uid).collect do |vm_uid|
        VM.new(vm_uid, self) # Isn't this expensive?
      end
    end
    
    # Find a certain VM by a given domain-name.
    def find_vm name
      vms.detect { |vm| vm.name == name } # || raise Exception
    end

    # Gives the different bridges on your system
    def networks
      get_value("network.get_all").collect do |network_uid|
        Network.new(network_uid, self)
      end
    end

    # Gives the API version number in the form of major.minor vendor.
    def version
      major  = get_value("host.get_API_version_major", uid)
      minor  = get_value("host.get_API_version_minor", uid)
      vendor = get_value("host.get_API_version_vendor", uid)

      "#{major}.#{minor} #{vendor}"
    end

    def record
      get_value("host.get_record")
    end

    # gives the local storage resource
    def get_local_sr
      SR.get_local(self)
    end

    def create_vdi(name, dev)
      VDI.create(name, dev, self)
    end

    def create_vbd(device, vm, vdi)
      VBD.create(device, vm, vdi, self)
    end

    def create_vif(vm, network, mac)
      VIF.create(vm, network, mac, self)
    end

    def create_vm(name, memory)
      VM.create(name, memory, self)
    end

    def create_console(vm)
      Console.create(vm, self)
    end

    def to_s
      uid.to_s
    end
    
    # Call to the API
    def call(function_name, *params)
      @connection.call(function_name, *params)
    end
    
    # Call to the API, Returns only the return value.
    def get_value(function, *params)
      call(function, *params)['Value']
    end
  end

end