module Xen
  class VDI < Base

    # The initial config for a new VDI device
    @initial_config = {
        :type => 'system', 
        :sharable => false, 
    }
  
    # Create method, takes a name, a device (p.ex. 'phy:/dev/storage/slice') and a host
    def self.create(name, dev, host)
      sr = host.get_local_sr
      config = @initial_config.merge({:name_label => name, 
                                          :other_config => {:location => dev}, 
                                          :SR => sr.uuid })
      uuid = host.get_value("VDI.create", config)
      self.new(uuid, host)
    end
    
    # Finds a device when given a devicename
    def self.find_by_device(dev, host)
      host.get_value('VDI.get_all').each do |vdi|
        vdirecord = host.get_value('VDI.get_record', vdi) 
        return self.new(vdi, host) if vdirecord['other_config']['location'] == dev
      end
      nil
    end
    
    # Destroys the current VDI
    def destroy!
      @host.call 'VDI.destroy', self.uuid
    end
    
    # Gets all the VDBs connected
    def vbds
      @host.get_value('VDI.get_VBDs', self.uuid).collect do |vbd|
        Xen::VBD.new(vbd, @host)
      end
    end

    # Gives some human string
    def humanize(prepend)
      r = self.record
      if !r 
        prepend + "Unknown vdi #{self.uuid}"
      else
        prepend + "VDI #{self.uuid} is device #{r['other_config']['location']}"
      end
    end
  end
end