module Xen

  class VM < Base

    @initial_config = {
      #    :name_label =>  '',
      :name_description =>  'Openminds APIs zijn vetgeil',
      :user_version =>  1,
      :is_a_template =>  false,
#      :memory_static_max =>  32,
#      :memory_dynamic_max =>  32,
#      :memory_dynamic_min =>  32,
#      :memory_static_min =>  32,
      :VCPUs_policy =>  '',
      :VCPUs_params =>  '',
      :VCPUS_features_required =>  '',
      :VCPUs_features_can_use =>  '',
      :VCPUs_features_force_on =>  '',
      :VCPUs_features_force_off =>  '',
      :actions_after_shutdown =>  'restart',
      :actions_after_reboot =>  'restart',
      :actions_after_suspend =>  'destroy',
      :actions_after_crash =>  'restart',
      :bios_boot =>  '',
      :platform_std_VGA =>  false,
      :platform_serial =>  '',
      :platform_localtime =>  false,
      :platform_clock_offset =>  false,
      :platform_enable_audio =>  false,
      :builder =>  'linux',
      :boot_method =>  '',
      :PV_kernel =>  '/boot/xenu-linux-2.6.18-domu',
      :PV_initrd =>  '',
      :PV_args =>  'root=/dev/hda1 ro 4',
      :grub_cmdline =>  '',
      :PCI_bus =>  '',
      :other_config =>  '' ,
    }

    # Creates a new VM with a certain name set and a amount of memory (in bytes) on a host
    def self.create(name, memory, host)
      o = @initial_config.merge({
        :name_label => name,
        :memory_static_max =>  memory,
        :memory_dynamic_max => memory,
        :memory_dynamic_min =>  memory,
        :memory_static_min =>  memory
      })
      uid = host.get_value("VM.create", o)
      self.new(uid, host)
    end

    # Creates a new VM, Given the VM's UID and the host it is running on.
    def initialize(uid, host)
      @uid, @host = uid, host
      preload_record!
    end

    # Gets the state of the VM. The first time it is called it query's the server, other times it is cached. There is an optional parameter 'refresh' which, when true, will re-query the server.
    # === Example
    #   # Having vm being a new initiated Vm
    #   vm.state # => "Running"
    #   vm.clean_shutdown! # => "Running"
    #   # We wait a while
    #   vm.state # => "Running" (cached)
    #   vm.state(true) # => "Halted"
    def state refresh = false
      @state = refresh || !@state ? @host.get_value("VM.get_power_state",@uid) : @state
    end

    # Returns the VM's name
    def name
      @name ||= @host.get_value("VM.get_name_label",@uid)
    end

    # Returns true if the current vm happens to be dom0.
    def is_dom0?
      @control_domain
    end
    
    # Starts a VM, and returns the current state
    def start!(paused = false)
      @host.call("VM.start",@uid, paused)    
      state(true)
    end

    # Pauses the VM, and returns the current state
    def pause!
      @host.call("VM.pause",@uid)    
      state(true)
    end

    # Unpauses the VM, and returns the current state
    def unpause!
      @host.call("VM.unpause",@uid)    
      state(true)
    end

    # Shuts down the VM in a clean way, and returns the current state
    def clean_shutdown!
      @host.call("VM.clean_shutdown",@uid)    
      state(true)
    end

    # Reboots the VM in a clean way, and returns the current state
    def clean_reboot!
      @host.call("VM.clean_reboot",@uid)    
      state(true)
    end

    # Shuts down the VM immediatly, hard, and returns the current state
    def hard_shutdown!
      @host.call("VM.hard_shutdown",@uid)    
      state(true)
    end

    # Hard reboots the VM, and returns the current state
    def hard_reboot!
      # Update 30/08/2007
      # This function just returns an ENOTIMPLEMENTED in the Xen API, so we'll have to implement it ourselves
      # @host.call("VM.hard_reboot",@uid)    
      
      # Should do:
        # hard_shutdown!
        # start!
      state(true)
    end
    
    def vdbs
      @host.get_value('VM.get_VBDs', self.uuid).collect do |vbd|
        Xen::VBD.new(vbd, @host)
      end
    end
    
    private
    def preload_record!
      data = record
      @dom_id = data["domid"]
      @kernel = data["PV_kernel"]
      @kernel_args = data["PV_args"]
      @name = data["name_label"]
      # vifs
      @control_domain = data["is_control_domain"]
    end
    
  end
end