#!/usr/bin/ruby -w
#
# $Id$
#
# The editor for Scotty TNG


require 'gtk2'
require 'cairo'
require 'libglade2'
require 'zlib'
require 'rexml/document'
require 'objects.rb'

begin
  Dir.chdir('/usr/share/scotty-tng')
rescue
end

class EditorGlade
  include GetText

  attr :glade

  def initialize(path_or_data, root = nil, domain = nil, localedir = nil, flag = GladeXML::FILE)
    bindtextdomain(domain, localedir, nil, "UTF-8")
    @glade = GladeXML.new(path_or_data, root, domain, localedir, flag) {|handler| method(handler)}
    @window_main = @glade.get_widget("window_main")
    @scrolledwindow_log = @glade.get_widget("scrolledwindow_log")
    @menu_view = @glade.get_widget("menu_view")

    # Show main window and startup splash
    Gtk::Window.auto_startup_notification = false
    @window_main.show
    Gtk::Window.auto_startup_notification = true

    @width = 320	# initial width of the view
    @height = 240	# initial height of the view

    # Gtk::Layout is used to draw the view
    @layout = @glade.get_widget("layout_main")
    @layout_img = nil
    @layout_ct = nil

    # Load all drawable objects
    exit if (@objects = EditorObjects.new("objects.xml")) == nil

    # Test
    w = @objects.to_widget("chart")
    @layout.add(w)

    w = @objects.to_widget("node")
    @layout.add(w)

    @layout.show_all
    
  end

  # Process pending Gtk events and return
  def iter
    while Gtk.events_pending? do
      Gtk.main_iteration
    end
  end


  # Initialize the Editor stuff
  def init_editor
    iter

    return 1
  end

  # Load a view from XML string
  def load_view(xview)
  end


  def on_open1_activate(widget)
    parent_window = widget.toplevel
    # TODO
    dialog = Gtk::FileChooserDialog.new("Open File",
					parent_window,
					Gtk::FileChooser::
					ACTION_OPEN,
					nil,
					[Gtk::Stock::CANCEL, Gtk::Dialog::
					  RESPONSE_CANCEL],
					[Gtk::Stock::OPEN, Gtk::Dialog::
					  RESPONSE_ACCEPT])

    
    if dialog.run == Gtk::Dialog::RESPONSE_ACCEPT
      #puts "filename = #{dialog.filename}"
      f = File.new(dialog.filename)
      str = String.new
      begin
	while (line = f.readline)
	  str << line.chomp
	end
      rescue EOFError
	f.close
      end

      #      begin
      xml = REXML::Document.new(str)
      
      xml.elements.each("/views/view") { |xview|
	@menu_view.append(Gtk::MenuItem.new(xview.attributes["title"]))
	load_view(xview)
      }
      @menu_view.show_all

      #      rescue
      #	Gtk.main_quit
      #      end
    end
    dialog.destroy
  end

  def on_save1_activate(widget)
    # TODO
  end

  def on_save_as1_activate(widget)
    # TODO
  end

  def on_delete1_activate(widget)
    # TODO
  end

  def on_new1_activate(widget)
    # TODO
  end

  def on_copy1_activate(widget)
    # TODO
  end

  def on_cut1_activate(widget)
    # TODO
  end

  def on_paste1_activate(widget)
    # TODO
  end

  def on_about1_activate(widget)
    # TODO
    dialog_about = Gtk::AboutDialog.new
    dialog_about.show
  end

  def on_quit1_activate(widget)
    # TODO
  end

  def button_exit_clicked_cb(widget)
    dialog = widget.toplevel
    dialog.hide
  end

  def window_main_delete_event_cb(widget, arg0)
    Gtk.main_quit
  end
end

if EditorGlade.new("editor.glade").init_editor != 0 then
  Gtk.main
end
